"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Campaign = void 0;
const pda_1 = require("../../utils/pda");
class Campaign {
    constructor(client, address, state, _onStateUpdate, _errorCallback) {
        this.client = client;
        this.address = address;
        this.state = state;
        this._onStateUpdate = _onStateUpdate;
        this._errorCallback = _errorCallback;
        _onStateUpdate && this.subscribe();
    }
    static async load(client, address, onStateUpdateHandler, errorCallback) {
        const state = (await client.accounts.campaign.fetch(address));
        return new Campaign(client, address, state, onStateUpdateHandler, errorCallback);
    }
    subscribe() {
        this.removeListener();
        try {
            this.addListener();
        }
        catch (error) {
            if (this._errorCallback) {
                this._errorCallback(error);
            }
        }
    }
    rewardVaultAddress() {
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        const [address, _] = (0, pda_1.deriveCampaignRewardVault)(this.address, this.client.programId);
        return address;
    }
    rewardVaultAuthorityAddress() {
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        const [address, _] = (0, pda_1.deriveCampaignRewardVaultAuthority)(this.address, this.client.programId);
        return address;
    }
    addListener() {
        this._listener = this.client.connection.onAccountChange(this.address, ({ data }) => {
            this.state = this.client.program.coder.accounts.decode('Campaign', data);
            if (this._onStateUpdate) {
                this._onStateUpdate(this.state);
            }
        }, 'processed');
    }
    removeListener() {
        if (this._listener)
            this.client.connection.removeAccountChangeListener(this._listener);
    }
    async unsubscribe() {
        this.removeListener();
    }
}
exports.Campaign = Campaign;
//# sourceMappingURL=campaign.js.map